# Generated by Django 3.2.7 on 2021-09-17 10:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0076_remove_reserved_resource"),
        ("rpm", "0039_disttree_digest"),
    ]

    operations = [
        migrations.CreateModel(
            name="RpmAlternateContentSource",
            fields=[
                (
                    "alternatecontentsource_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="rpm_rpmalternatecontentsource",
                        serialize=False,
                        to="core.alternatecontentsource",
                    ),
                ),
            ],
            options={
                "default_related_name": "%(app_label)s_%(model_name)s",
            },
            bases=("core.alternatecontentsource",),
        ),
    ]
